module pimod
double precision pi2
end module

program main

!   This demonstrates a potential numerical inaccuracy in the context of
!   using Simpson's rule to compute the value of a definite integral.
!   David H Bailey    2008-01-09

use pimod
implicit none
integer k, n
parameter (n = 1000000)
double precision a, b, fun, h, s1, x
external fun

a = 0.d0
b = 1.d0
pi2 = 0.5d0 * acos (-1.d0)
h = (b - a) / (2.d0 * n)
write (6, '(1p,d25.16)') h
x = a
s1 = fun (x)

do k = 1, n
  x = a + (2 * k - 1) * h
  s1 = s1 + 4.d0 * fun (x)
  if (k == n) goto 100
  x = x + h
  s1 = s1 + 2.d0 * fun (x)
enddo

100 continue

x = x + h
s1 = s1 + fun (x)
write (6, '(1p,d25.16)') x, h * s1 / 3.d0
stop
end

function fun (x)
use pimod
double precision fun, x

fun = sin (pi2 * x)
return
end
