module pimod
double precision pi
end module

subroutine f_main

!   This demonstrates a potential numerical inaccuracy in the context of
!   using Simpson's rule to compute the value of a definite integral.
!   This code uses the QD package, available from 
!   http://crd.lbl.gov/~dhbailey/mpdist

!   David H Bailey    2008-01-09

use ddmodule
use pimod
implicit none
integer i, j, k, n
parameter (n = 1000000)
double precision a, b, fun
type (dd_real) h, s1, x
double precision fuzz
parameter (fuzz = 1d-26)
external fun
integer*4 old_cw

call f_fpu_fix_start (old_cw)
a = 0.d0
b = 1.d0
pi = acos (-1.d0)
h = (b - a) / (2.d0 * n)
write (6, '(1p,d25.16)') dble (h)
x = a
s1 = fun (x)

100 continue

x = x + h
s1 = s1 + 4.d0 * fun (dble (x))
x = x + h
if (x + fuzz >= b) goto 110
s1 = s1 + 2.d0 * fun (dble (x))
goto 100

110 continue

s1 = s1 + fun (x)
write (6, '(1p,d25.16)') dble (x), dble (h * s1 / 3.d0)
stop
end

function fun (x)
use pimod
double precision fun, x

fun = sin (pi * x)
return
end
